import six
from six.moves import UserDict


class PolicyConfig(UserDict):
    @property
    def roles(self):
        return self.data.get('roles', {})

    @property
    def groups(self):
        return self.data.get('groups', {})

    @property
    def routes(self):
        return self.data.get('routes', {})

    @property
    def route_policies(self):
        for route, route_cfg in six.iteritems(self.routes):
            for method, policy in six.iteritems(route_cfg):
                yield route, method, policy



import falcon

from falcon_policy.config import PolicyConfig
from falcon_policy.policy import PolicyManager


class RoleBasedPolicy(object):
    def __init__(self, config_dict):
        self.config = PolicyConfig(config_dict)
        self.manager = PolicyManager(self.config)

    def process_resource(self, req, resp, resource, params):
        route = req.uri_template

        if isinstance(req.context, dict):
            roles = req.context.get('roles')

        # Assume it's a direct attribute
        else:
            roles = getattr(req.context, 'roles', None)

        # Fallback on the X-Roles Header
        if not roles:
            roles_header = req.get_header('X-Roles', default='@unknown')
            roles = [role.strip() for role in roles_header.split(',')]

        route_policy = self.manager.policies.get(route, {})
        method_policy = route_policy.get(req.method.upper(), [])

        has_role = self.manager.check_roles(roles, method_policy)

        if not has_role:
            raise falcon.HTTPForbidden(
                description='Access to this resource has been restricted'
            )



#!/usr/bin/env python

from setuptools import setup, find_packages

desc = ''
with open('README.rst') as f:
    desc = f.read()

setup(
    name='falcon-policy',
    version='0.2.0',
    description=('Policy middleware for Falcon APIs'),
    long_description=desc,
    url='https://github.com/falconry/falcon-policy',
    author='John Vrbanac',
    author_email='john.vrbanac@linux.com',
    license='Apache v2',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='falcon middleware policy rbac role based access',
    packages=find_packages(exclude=['contrib', 'docs', 'test*']),
    install_requires=[
        'falcon',
        'six',
    ],
    package_data={},
    data_files=[],
    entry_points={
        'console_scripts': [],
    },
)



import collections
import itertools


class PolicyManager(object):
    def __init__(self, config):
        self.config = config
        self.policies = collections.defaultdict(dict)

        self.cache_expanded_policies()

    def expand_policy(self, policy):
        to_expand = [
            item for item in policy
            if item in self.config.groups
        ]

        group_roles = itertools.chain.from_iterable([
            self.config.groups[item] for item in to_expand
        ])

        roles = set(policy) - set(to_expand)
        authorized_roles = roles.union(set(group_roles))

        return authorized_roles

    def cache_expanded_policies(self):
        for route, method, policy in self.config.route_policies:
            self.policies[route][method.upper()] = self.expand_policy(policy)

    def check_roles(self, provided_roles, policy):
        authorized_list = policy

        if '@any-role' in policy:
            authorized_list = self.config.roles

        elif '@passthrough' in policy:
            return next(iter(provided_roles))

        for provided in provided_roles:
            role = provided.strip()

            if role in authorized_list:
                return role
